#include  "sockctl_option_analyzer.h"
#include  "sockctl.h"
#include  "socket_io.h"
#include  "combination_io.h"
#include  "fd_io.h"
#include  "file_io.h"
#include  "comstream.h"
#if defined(ADD_BINARY_SIZE_HEADER)
#  include  "line_buffer_binary_size_wrapper_output.h"
#endif
#include  <fcntl.h>		// for open()
#include  <csignal>		// for signal()
#include  <cstdio>		// for perror()
#include  <exception>
#include  <iostream>

#ifndef DEFAULT_PORT
#  define DEFAULT_PORT 0
#endif


int    main( int  argc ,  char *  argv[] )
{
  try
  {
	//
	// option analysis
	//
	Sockctl_Option_Analyzer	opt( "127.0.0.1" /* IPv4 localhost */ ,
				     DEFAULT_PORT );

	if ( ! opt.analyze( argc , argv ) )
	{
		opt.usage();
		return( 1 );
	}
	else if ( opt.help() )
	{
		opt.usage();
		return( 0 );
	}


	//
	// signal handling
	//
	if ( std::signal( SIGPIPE , SIG_IGN ) == SIG_ERR )
	{
		std::perror( "signal" );

		return( 1 );
	}


	if ( opt.rest_options().empty() )
	{
#if !defined(ADD_BINARY_SIZE_HEADER)
		Sockctl	sockctl( new Socket_IO( opt.host() , opt.port() ,
						opt.ipv4_only() ) ,
				 new Combination_IO
					( new FD_IO( STDIN_FILENO  ) ,
					  new FD_IO( STDOUT_FILENO ) ) ,
				 opt.debug() );
#else
		ref_count_ptr<Abstract_IO>	s;
		s = new Socket_IO( opt.host() , opt.port() , opt.ipv4_only() );

		Sockctl	sockctl( new Combination_IO
				 ( s , new Line_Buffer_Binary_Size_Wrapper_Output( s ) ) ,
				 new Combination_IO
					( new FD_IO( STDIN_FILENO  ) ,
					  new FD_IO( STDOUT_FILENO ) ) ,
				 opt.debug() );
#endif

		sockctl.run();
	}
	else
	{
		for ( size_t  i = 0  ;  i < opt.rest_options().size()  ;
		      i ++ )
		{
#if 0
			icomstream	com( "grep" ,
					     "-v" ,
					     "^#" ,
					     opt.rest_options()[i].c_str() ,
					     static_cast<char *>(0) );

			Sockctl	sockctl( new Socket_IO
					    ( opt.host() , opt.port() ,
					      opt.ipv4_only() ) ,
					 new Combination_IO
					    ( new FD_IO
						  ( com.fd( std::ios::out ) ) ,
					      new FD_IO( STDOUT_FILENO ) ) ,
					 opt.debug() );
#elif ! defined(ADD_BINARY_SIZE_HEADER)
			Sockctl	sockctl( new Socket_IO( opt.host() ,
							opt.port() ,
							opt.ipv4_only() ) ,
					 new Combination_IO
					 ( new File_IO( opt.rest_options()[i] ,
							/* read  */ true ,
							/* write */ false ) ,
					   new FD_IO( STDOUT_FILENO ) ) ,
					 opt.debug() );
#else
			ref_count_ptr<Abstract_IO>	s;
			s = new Socket_IO( opt.host() , opt.port() ,
					   opt.ipv4_only() );

			Sockctl	sockctl( new Combination_IO
					 ( s ,
					   new Line_Buffer_Binary_Size_Wrapper_Output( s ) ) ,
					 new Combination_IO
					 ( new File_IO( opt.rest_options()[i] ,
							/* read  */ true ,
							/* write */ false ) ,
					   new Line_Buffer_Binary_Size_Wrapper_Output
					   ( new FD_IO( STDOUT_FILENO ) ) ) ,
					 opt.debug() );
#endif

			sockctl.run();
		}
	}
  }
  catch( const std::exception &  e )
  {
	if ( *(e.what()) != '\0' )
	{
		std::cerr << e.what() << std::endl;
	}

	return( 1 );
  }
  catch(...)
  {
	std::cerr << "exception caught!!" << std::endl;

	return( 1 );
  }

  return( 0 );
}
